// Active Notifications Indicator Extension for the Gnome Shell
//
// active-notifications-indicator@nls1729
//
// This extension is a derived work of the Gnome Shell.
//
// Copyright (c) 2012 Norman L. Smith
//
// This extension is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This extension is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with the Gnome Shell; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// Version 2 - fixed bug to correct indicator to scale properly when gnome tweak tool is used to adjust scale fonts.

const Lang = imports.lang;
const St = imports.gi.St;

const Main = imports.ui.main;
const MessageTray = imports.ui.messageTray;
const TelepathyClient = imports.ui.telepathyClient

let notificationsIndicator;

function NotificationsIndicator() {
    this._init();
}

NotificationsIndicator.prototype = {

    _init: function() {
        this._summaryItems = [];

        this._indicatorBox = new St.BoxLayout({name: 'active-notifications-box',
                                               style_class: 'active-notifications-box'});
        this._indicatorCount = new St.Label();

        this._indicatorCountBin = new St.Bin({style_class: 'counter-blue',
                                              child: this._indicatorCount});

        this._indicatorChat = new St.Icon({icon_name: 'avatar-default',
                                           icon_type: St.IconType.FULLCOLOR,
                                           style_class: 'system-status-icon'});

        this._indicatorNotification = new St.Icon({icon_name: 'gtk-dialog-info',
                                                   icon_type: St.IconType.FULLCOLOR,
                                                   style_class: 'system-status-icon'});

        this._indicatorIconBin = new St.Bin({x_fill: true,
                                             y_fill: true});

        this._indicatorBox.add(this._indicatorCountBin);
        this._indicatorBox.add(this._indicatorIconBin);
        this._resetIndicator();
    },

    _resetIndicator: function() {
        this._indicatorCountBin.style_class = 'counter-blue';
        this._indicatorBox.opacity = 120;
        this._indicatorIconBin.child = this._indicatorNotification;
        this._indicatorCount.text = '';
    },

    _setIndicator: function() {
        if (this._summaryItems.length == 0) {
            this._resetIndicator();
        } else {
            this._indicatorBox.opacity = 250;
            let chat = false;
            let color = 'counter-blue';
            for (let i = 0; i < this._summaryItems.length; i++) {
                let item = this._summaryItems[i];
                if (item.source.isChat) {
                    chat = true;
                    if (item.source._pendingMessages.length > 0) {
                        color = 'counter-red';
                        break;
                    }
                }
            }
            if (this._summaryItems.length > 0)
                this._indicatorCount.text = this._summaryItems.length.toString();
            else
                this._indicatorCount.text = '';
            if (!chat)
                this._indicatorIconBin.child = this._indicatorNotification;
            else
                this._indicatorIconBin.child = this._indicatorChat;
            this._indicatorCountBin.style_class = color;
        }
    },

    enable: function() {
        let chatSource = TelepathyClient.ChatSource.prototype;
        let source = MessageTray.Source.prototype;
        let tray = MessageTray.MessageTray.prototype;
        chatSource._originalMessageSent = chatSource._messageSent;
        chatSource._originalSetCount = chatSource._setCount;
        source._originalSetCount = source._setCount;
        tray._originalUpdateState = tray._updateState;

        MessageTray.Source.prototype._setCount = function(count, visible) {
            this._originalSetCount(count, visible);
            notificationsIndicator._setIndicator();
        }
        TelepathyClient.ChatSource.prototype._messageSent = function(channel, message, flags, token) {
            this._originalMessageSent(channel, message, flags, token);
            notificationsIndicator._indicatorCountBin.style_class = 'counter-blue';
        }
        TelepathyClient.ChatSource.prototype._setCount = function(count, visible) {
            if (visible) {
                notificationsIndicator._indicatorCountBin.style_class = 'counter-red';
                this._counterBin.style_class = 'counter-red';
            }
            this._originalSetCount(count, visible);
            notificationsIndicator._setIndicator();
        }
        MessageTray.MessageTray.prototype._updateState = function() {
            this._originalUpdateState();
            notificationsIndicator._summaryItems = this._summaryItems;
        }

        Main.panel._rightBox.insert_actor(notificationsIndicator._indicatorBox, -1);
    },

    disable: function() {
        let chatSource = TelepathyClient.ChatSource.prototype;
        let source = MessageTray.Source.prototype;
        let tray = MessageTray.MessageTray.prototype;
        chatSource._messageSent = chatSource._originalMessageSent;
        chatSource._setCount = chatSource._originalSetCount;
        source._setCount = source._originalSetCount;
        tray._updateState = tray._originalUpdateState;
        Main.panel._rightBox.remove_actor(notificationsIndicator._indicatorBox);
        chatSource._originalMessageSent = null;
        chatSource._originalSetCount = null;
        source._originalSetCount = null;
        tray._originalUpdateState = null;
    }
};

function init() {
    notificationsIndicator = new NotificationsIndicator();
    return notificationsIndicator;
}
